package com.agilecontrol.nea.core.cxtab;

import com.agilecontrol.nea.core.query.QueryEngine;
import com.agilecontrol.nea.core.query.QueryUtils;
import com.agilecontrol.nea.core.security.User;
import com.agilecontrol.nea.core.util.ConfigValues;
import com.agilecontrol.nea.util.CommandExecuter;
import com.agilecontrol.nea.util.NDSException;
import com.agilecontrol.nea.util.Validator;
import java.io.File;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.Date;
import org.json.JSONArray;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class SQLiteDB {

   private static Logger logger = LoggerFactory.getLogger(SQLiteDB.class);
   private int piId;
   private int dimCount;
   private String file;
   private String sql;
   private int cxtabId;
   private String filterDesc;
   private String cxtabName;
   private User user;
   private String dataSQL;
   private Connection oraConn;


   public SQLiteDB(int piId, int dimensions, String sql, String file, int cxtabId, String cxtabName, String filterDesc, User user, Connection conn) {
      this.piId = piId;
      this.file = file;
      this.sql = sql;
      this.oraConn = conn;
      this.dimCount = dimensions;
      this.filterDesc = filterDesc;
      this.cxtabName = cxtabName;
      this.user = user;
      this.cxtabId = cxtabId;
   }

   public static void main(String[] args) throws ClassNotFoundException {
      Class.forName("org.sqlite.JDBC");
      Connection connection = null;

      try {
         connection = DriverManager.getConnection("jdbc:sqlite:e:/s.db");
         Statement e = connection.createStatement();
         e.setQueryTimeout(30);
         e.executeUpdate("insert into person values(1, \'朱叶峰\')");
         ResultSet rs = e.executeQuery("select * from person");

         while(rs.next()) {
            System.out.println("name = " + rs.getString("name"));
            System.out.println("id = " + rs.getInt("id"));
         }
      } catch (SQLException var12) {
         System.err.println(var12.getMessage());
      } finally {
         try {
            if(connection != null) {
               connection.close();
            }
         } catch (SQLException var11) {
            System.err.println(var11);
         }

      }

   }

   public int save() throws Exception {
      Class.forName("org.sqlite.JDBC");
      Statement stmt = null;
      Connection conn = null;
      boolean cnt = false;
      Object tmpFile = null;

      int cnt1;
      try {
         conn = DriverManager.getConnection("jdbc:sqlite:");
         stmt = conn.createStatement();
         stmt.executeUpdate("PRAGMA synchronous = OFF;");
         cnt1 = this.migrateData(conn);
         this.migrateDims(conn);
         this.migrateFacts(conn);
         this.migrateCubeInfo(conn, cnt1);
         logger.debug("backup to " + this.file + ".sdb");
         stmt.executeUpdate("backup to \"" + this.file + ".sdb\"");
      } finally {
         if(stmt != null) {
            try {
               stmt.close();
            } catch (Throwable var14) {
               ;
            }
         }

         if(conn != null) {
            try {
               conn.close();
            } catch (Throwable var13) {
               ;
            }
         }

      }

      this.compressFile();
      return cnt1;
   }

   private void compressFile() throws Exception {
      String z7r = ConfigValues.get("program.7zr", ConfigValues.get("dir.nea.root") + "/bin/7zr.exe");
      String log = this.file + ".log";
      CommandExecuter cmd = new CommandExecuter(log);
      File sdbFile = new File(this.file + ".sdb");
      if(!sdbFile.exists()) {
         throw new NDSException(sdbFile.getAbsolutePath() + " not find");
      } else {
         if(z7r.endsWith(".sh")) {
            cmd.run(z7r + " " + this.file + " " + this.file + ".sdb");
         } else {
            cmd.run(z7r + " a \"" + this.file + "\" \"" + this.file + ".sdb\"");
         }

         if(!(new File(this.file)).exists()) {
            throw new NDSException("Report creation failed, check " + this.file + ".log on server");
         } else {
            File f = new File(this.file + ".sdb");
            f.delete();
            f = new File(log);
            f.delete();
         }
      }
   }

   private void migrateCubeInfo(Connection conn, int cnt) throws Exception {
      Statement stmt = null;
      PreparedStatement pstmt = null;

      try {
         stmt = conn.createStatement();
         stmt.executeUpdate("create table ad_cxtab(piid, name, filter, owner, creationdate,rowcnt,usr_sgrade, sql)");
         pstmt = conn.prepareStatement("insert into ad_cxtab(piid, name, filter, owner, creationdate,rowcnt,usr_sgrade,sql)values(?,?,?,?,?,?,?,?)");
         pstmt.setInt(1, this.piId);
         pstmt.setString(2, this.cxtabName);
         pstmt.setString(3, this.filterDesc);
         pstmt.setString(4, this.user.getName());
         pstmt.setString(5, ((SimpleDateFormat)QueryUtils.dateTimeSecondsFormatter.get()).format(new Date()));
         pstmt.setInt(6, cnt);
         pstmt.setInt(7, this.user.getSecurityGrade());
         pstmt.setString(8, this.dataSQL);
         pstmt.executeUpdate();
      } finally {
         if(stmt != null) {
            try {
               stmt.close();
            } catch (Throwable var14) {
               ;
            }
         }

         if(pstmt != null) {
            try {
               pstmt.close();
            } catch (Throwable var13) {
               ;
            }
         }

      }

   }

   private void migrateFacts(Connection conn) throws Exception {
      Statement stmt = null;
      PreparedStatement pstmt = null;

      try {
         JSONArray list = QueryEngine.getInstance().doQueryJSONArray("select id, description, function_, userfact,valuename,valueformat,sgrade,orderno from ad_cxtab_fact where ad_cxtab_id=? and isactive=\'Y\'", new Object[]{Integer.valueOf(this.cxtabId)}, this.oraConn);
         stmt = conn.createStatement();
         stmt.executeUpdate("create table ad_cxtab_fact(id, description, function_, userfact,valuename,valueformat,sgrade,orderno,isactive)");
         stmt.close();
         pstmt = conn.prepareStatement("insert into ad_cxtab_fact(id, description, function_, userfact,valuename,valueformat,sgrade,orderno,isactive)values(?,?,?,?,?,?,?,?,\'Y\')");

         for(int i = 0; i < list.length(); ++i) {
            JSONArray row = list.getJSONArray(i);
            pstmt.setInt(1, row.getInt(0));
            if(row.isNull(1)) {
               pstmt.setNull(2, 12);
            } else {
               pstmt.setString(2, row.getString(1));
            }

            if(row.isNull(2)) {
               pstmt.setNull(3, 12);
            } else {
               pstmt.setString(3, row.getString(2));
            }

            if(row.isNull(3)) {
               pstmt.setNull(4, 12);
            } else {
               pstmt.setString(4, row.getString(3));
            }

            if(row.isNull(4)) {
               pstmt.setNull(5, 12);
            } else {
               pstmt.setString(5, row.getString(4));
            }

            if(row.isNull(5)) {
               pstmt.setNull(6, 12);
            } else {
               pstmt.setString(6, row.getString(5));
            }

            pstmt.setInt(7, row.getInt(6));
            pstmt.setInt(8, row.getInt(7));
            pstmt.executeUpdate();
         }
      } finally {
         if(stmt != null) {
            try {
               stmt.close();
            } catch (Throwable var15) {
               ;
            }
         }

         if(pstmt != null) {
            try {
               pstmt.close();
            } catch (Throwable var14) {
               ;
            }
         }

      }

   }

   private void migrateDims(Connection conn) throws Exception {
      Statement stmt = null;
      PreparedStatement pstmt = null;

      try {
         JSONArray list = QueryEngine.getInstance().doQueryJSONArray("select id, description, position_, hidehtml,orderno  from ad_cxtab_dimension where ad_cxtab_id=? and isactive=\'Y\'", new Object[]{Integer.valueOf(this.cxtabId)}, this.oraConn);
         String userPrefIds = (String)QueryEngine.getInstance().doQueryOne("select value from ad_user_pref where ad_user_id=? and module=\'cxtabdim\' and name=?", new Object[]{Integer.valueOf(this.user.getId().intValue()), String.valueOf(this.cxtabId)}, this.oraConn);
         boolean hasUserPrefIds = false;
         if(Validator.isNotNull(userPrefIds)) {
            userPrefIds = userPrefIds + ",";
            hasUserPrefIds = true;
         }

         stmt = conn.createStatement();
         stmt.executeUpdate("create table ad_cxtab_dimension(id,description, position_, hidehtml,orderno,isactive)");
         stmt.close();
         pstmt = conn.prepareStatement("insert into ad_cxtab_dimension(id,description, position_, hidehtml,orderno,isactive)values(?,?,?,?,?,\'Y\')");

         for(int i = 0; i < list.length(); ++i) {
            JSONArray row = list.getJSONArray(i);
            int id = row.getInt(0);
            if(hasUserPrefIds && userPrefIds.contains(id + ",") || !hasUserPrefIds) {
               pstmt.setInt(1, row.getInt(0));
               if(row.isNull(1)) {
                  pstmt.setNull(2, 12);
               } else {
                  pstmt.setString(2, row.getString(1));
               }

               if(row.isNull(2)) {
                  pstmt.setNull(3, 12);
               } else {
                  pstmt.setString(3, row.getString(2));
               }

               if(row.isNull(3)) {
                  pstmt.setNull(4, 12);
               } else {
                  pstmt.setString(4, row.getString(3));
               }

               pstmt.setInt(5, row.getInt(4));
               pstmt.executeUpdate();
            }
         }
      } finally {
         if(stmt != null) {
            try {
               stmt.close();
            } catch (Throwable var18) {
               ;
            }
         }

         if(pstmt != null) {
            try {
               pstmt.close();
            } catch (Throwable var17) {
               ;
            }
         }

      }

   }

   private int migrateData(Connection conn) throws Exception {
      ResultSet rs = null;
      Statement stmt = null;
      PreparedStatement pstmt = null;

      int i;
      try {
         rs = this.oraConn.createStatement().executeQuery(this.sql);
         ResultSetMetaData rsmd = rs.getMetaData();
         int colcnt = rsmd.getColumnCount();
         conn.setAutoCommit(false);
         stmt = conn.createStatement();
         StringBuilder createTableSQL = new StringBuilder("create table ad_cxtab_data(");
         StringBuilder insertSQL = new StringBuilder("insert into ad_cxtab_data(");
         StringBuilder qsql = new StringBuilder("select ");
         createTableSQL.append(rsmd.getColumnName(1));
         insertSQL.append(rsmd.getColumnName(1));
         qsql.append(rsmd.getColumnName(1));

         int obj;
         for(obj = 2; obj <= colcnt; ++obj) {
            createTableSQL.append(",").append(rsmd.getColumnName(obj));
            insertSQL.append(",").append(rsmd.getColumnName(obj));
            qsql.append(",").append(rsmd.getColumnName(obj));
         }

         createTableSQL.append(")");
         qsql.append(" from ad_cxtab_data");
         this.dataSQL = qsql.toString();
         insertSQL.append(") values(?");

         for(obj = 2; obj <= colcnt; ++obj) {
            insertSQL.append(",?");
         }

         insertSQL.append(")");
         logger.debug(createTableSQL.toString());
         stmt.executeUpdate(createTableSQL.toString());
         logger.debug(insertSQL.toString());
         pstmt = conn.prepareStatement(insertSQL.toString());
         int cnt = 0;
         short BATCH_COUNT = 5000;

         while(rs.next()) {
            for(i = 1; i <= this.dimCount; ++i) {
               String var30 = rs.getString(i);
               if(rs.wasNull()) {
                  pstmt.setString(i, "");
               } else {
                  pstmt.setString(i, var30);
               }
            }

            for(i = this.dimCount + 1; i <= colcnt; ++i) {
               double dbl = rs.getDouble(i);
               if(rs.wasNull()) {
                  pstmt.setNull(i, 8);
               } else {
                  pstmt.setDouble(i, dbl);
               }
            }

            pstmt.addBatch();
            ++cnt;
            if(cnt % BATCH_COUNT == 0) {
               pstmt.executeBatch();
            }
         }

         pstmt.executeBatch();
         conn.commit();
         conn.setAutoCommit(true);
         QueryEngine.getInstance().executeUpdate("update ad_pinstance_para set p_number=? where ad_pinstance_id=? and name=\'filename\'", new Object[]{Integer.valueOf(cnt), Integer.valueOf(this.piId)}, this.oraConn);
         i = cnt;
      } finally {
         if(rs != null) {
            try {
               rs.close();
            } catch (Throwable var28) {
               ;
            }
         }

         if(stmt != null) {
            try {
               stmt.close();
            } catch (Throwable var27) {
               ;
            }
         }

         if(pstmt != null) {
            try {
               pstmt.close();
            } catch (Throwable var26) {
               ;
            }
         }

      }

      return i;
   }

}
